<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.1"
  xmlns:xlink="http://www.w3.org/1999/xlink"
  xmlns:fo="http://www.w3.org/1999/XSL/Format"
  xmlns:d="http://docbook.org/ns/docbook"
  xmlns="http://www.w3.org/1999/xhtml">
  
  <xsl:import href="../../docbook-xsl/fo/docbook.xsl"/>  
  
  <!-- 
    <xsl:import href="../../docbook-xsl/fo/highlight.xsl"/>
    
  -->
  <xsl:import href="../pdf/highlight.xsl"/>
  
  <xsl:import href="../Common/path.xsl"/>    
  <xsl:import href="../../Preprocess/Xsl/slideReference.xsl"/>
  
  <xsl:param name="highlight.source" select="1"/>
  <xsl:param name="use.extensions" select="1"/>
  <xsl:param name="fop1.extensions" select="1"/>
  <xsl:param name="tablecolumns.extension" select="1"/>
  
  <xsl:output indent="yes"/>
  
  <xsl:variable name="bookmarks.state">
    <xsl:choose>
      <xsl:when test="$bookmarks.collapse != 0">hide</xsl:when>
      <xsl:otherwise>show</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  
  <xsl:template match="/">        
    <xsl:apply-templates select="
      (//d:chapter | //d:section | //d:appendix) [contains(@annotations, 'slide') or 
      descendant::*[contains(@annotations, 'slide')]]"
      mode = "slideUnit"/>    
  </xsl:template>
  
  <xsl:template match="d:chapter | d:section | d:appendix" mode="slideUnit">
    
    <xsl:variable name="presentationFoFileName">
      <xsl:call-template name="presentationFoFilename">
        <xsl:with-param name="slideContainerId" select="@xml:id"/>
      </xsl:call-template>
    </xsl:variable>
    
    
    <!-- Writing bigBlueButton FO --> 
    <xsl:document href="{$presentationFoFileName}" method="xml" indent="no">
      
      
      <xsl:variable name="revealSections" select="          
        .//d:figure[not (ancestor::d:qandaset)] |
        
        (descendant-or-self::d:chapter|
        descendant-or-self::d:section |
        descendant-or-self::d:appendix) [d:figure[not (ancestor::d:qandaset)]]"/>
      
      <fo:root>
        
        <fo:layout-master-set>
          <fo:simple-page-master master-name="simplePageLayout"
            page-width  = "294mm" page-height   = "150mm"
            margin-top  = "3mm"  margin-bottom = "2mm"
            margin-left = "4mm"  margin-right  = "4mm">
            
            <fo:region-body margin-top  = "12mm" margin-bottom = "0mm" 
              margin-left = "0mm" margin-right  = "0mm"/>
            <fo:region-before extent="7mm"/> 
          </fo:simple-page-master>
        </fo:layout-master-set>
        
        <xsl:apply-templates select="$revealSections" mode="slideRootSections"/>
      </fo:root>
      
    </xsl:document>
  </xsl:template>
  
  <xsl:template name="generateFollowupQandasetHeaders">
    
    <xsl:variable name="currentFigureGenerateId" select="generate-id()"/>    
    <xsl:variable name="followupQandasets" 
      select="following::d:qandaset[generate-id(preceding::d:figure[not(ancestor::d:qandaset)][1]) = $currentFigureGenerateId]"/>
    
    <xsl:if test="$followupQandasets">
      <fo:block>
        <xsl:choose>
          <xsl:when test="1 = count($followupQandasets)">
            <h3>Followup exercise</h3>
          </xsl:when>
          <xsl:otherwise>
            <h3>Followup exercises</h3>
          </xsl:otherwise>
        </xsl:choose>            
      </fo:block>      
    </xsl:if>   
  </xsl:template>
  
  <xsl:template match="d:chapter|d:section|d:appendix" mode="slideRootSections">
    
    <xsl:comment>
      <xsl:text>id=</xsl:text>
      <xsl:value-of select="@xml:id"/>
      <xsl:text>, title=</xsl:text>
      <xsl:value-of select="d:title"/>
    </xsl:comment>
  </xsl:template>
  
  <xsl:template match="d:figure" mode="slideRootSections">
    
    <xsl:variable name="primarySectionId">
      <xsl:apply-templates mode="id_or_generate_id" select="."/>
    </xsl:variable>
    
    <xsl:variable name="figureTitle" >
      <xsl:apply-templates select="d:title"/>
    </xsl:variable>
    
    <xsl:variable name="baseSupplementDir">
      <xsl:if test="ancestor::*/@xml:base">
        <xsl:call-template name="hdm.get.leftmost.of.separator">
          <xsl:with-param name="path" select="ancestor::*/@xml:base"/>
          <xsl:with-param name="separator" select="'/'"/>
        </xsl:call-template>
        <xsl:text>/</xsl:text>
      </xsl:if>
    </xsl:variable>
    
    <xsl:variable name="figBasename">
      <xsl:call-template name="hdm.get.leftmost.of.separator">
        <xsl:with-param name="path" select="d:mediaobject/d:imageobject/d:imagedata/@fileref"/>
        <xsl:with-param name="separator" select="'.fig.svg'"/>
      </xsl:call-template>
      <xsl:call-template name="hdm.get.leftmost.of.separator">
        <xsl:with-param name="path" select="d:mediaobject/d:imageobject/d:imagedata/@fileref"/>
        <xsl:with-param name="separator" select="'.multi.svg'"/>
      </xsl:call-template>
    </xsl:variable>      
    
    <xsl:variable name="figSet" as="element()*">
      <xsl:if test="0 &lt; string-length($figBasename)">
        <xsl:copy-of select="document(concat($figBasename, '.filenames'), .)/start/file"/>
      </xsl:if>
    </xsl:variable>
    
    <xsl:variable name="titleNode" select="d:title"/>
      
    
    <xsl:choose>
      
      <xsl:when test="1 &lt; count($figSet/*)"> <!-- Image series -->
        
        <xsl:for-each select="$figSet/*">
          <fo:page-sequence master-reference="simplePageLayout">
            <fo:static-content flow-name="xsl-region-before"> 
              <fo:block font-size="xx-large" color="red" wrap-option="no-wrap" overflow="hidden">
                <xsl:apply-templates select="$titleNode/text()|$titleNode/*" text-align-last='justify'/>
              </fo:block>
              <fo:block font-size="xx-small" color="red" text-align-last='justify'>
                <fo:leader leader-pattern="rule" leader-length="100%"/>
              </fo:block>
            </fo:static-content>
            <fo:flow flow-name="xsl-region-body">
              
              <fo:block>
                <fo:external-graphic src="{concat($baseSupplementDir, $figBasename, '-', position() - 1, '.svg')}"/>
              </fo:block>
            </fo:flow>
          </fo:page-sequence>
        </xsl:for-each>     
      </xsl:when>
      
      <xsl:when test="d:mediaobject/d:imageobject/d:imagedata/@fileref"> <!-- Single image -->
        <fo:page-sequence master-reference="simplePageLayout">
          <fo:static-content flow-name="xsl-region-before"> 
            <fo:block font-size="xx-large" color="red" wrap-option="no-wrap" overflow="hidden">
              <xsl:apply-templates select="$titleNode/text()|$titleNode/*" text-align-last='justify'/>
            </fo:block>
            <fo:block font-size="xx-small" color="red" text-align-last='justify'>
              <fo:leader leader-pattern="rule" leader-length="100%"/>
            </fo:block>
          </fo:static-content>
          <fo:flow flow-name="xsl-region-body">
            <fo:block>
              <fo:external-graphic>
                <xsl:attribute name="src">
                  <xsl:value-of select="$baseSupplementDir"/>
                  <xsl:value-of select="d:mediaobject/d:imageobject/d:imagedata/@fileref"/>
                </xsl:attribute>
              </fo:external-graphic>
            </fo:block>
            
          </fo:flow>
        </fo:page-sequence>
      </xsl:when>
      
      <xsl:otherwise>          <!-- Other content -->
        <fo:page-sequence master-reference="simplePageLayout">
          <fo:static-content flow-name="xsl-region-before"> 
            <fo:block font-size="xx-large" color="red" wrap-option="no-wrap" overflow="hidden">
              <xsl:apply-templates select="$titleNode/text()|$titleNode/*" text-align-last='justify'/>
            </fo:block>
            <fo:block font-size="xx-small" color="red" text-align-last='justify'>
              <fo:leader leader-pattern="rule" leader-length="100%"/>
            </fo:block>
          </fo:static-content>
          
          <fo:flow flow-name="xsl-region-body">
            
            <xsl:apply-templates select="*[not(self::d:title)]"/>
          </fo:flow>
        </fo:page-sequence>
      </xsl:otherwise>             
    </xsl:choose>
    
    <xsl:call-template name="generateFollowupQandasets"/>
    
  </xsl:template>
  
  <xsl:template name="generateFollowupQandasets">
    
    <xsl:variable name="currentFigureGenerateId" select="generate-id()"/>    
    <xsl:variable name="followupQandasets" 
      select="following::d:qandaset[generate-id(preceding::d:figure[not(ancestor::d:qandaset)][1]) = $currentFigureGenerateId]"/>
    
    <xsl:if test="$followupQandasets">
      
      <xsl:if test="10 &lt; count($followupQandasets)">
        <xsl:message>
          <xsl:text>Warning: </xsl:text>
          <xsl:value-of select="count($followupQandasets)"/>
          <xsl:text> consecutive followup exercises at slide '</xsl:text>
          <xsl:apply-templates select="d:title"/>
          <xsl:text>', id=</xsl:text>
          <xsl:value-of select="@xml:id"/>
        </xsl:message>
      </xsl:if>
      
      <xsl:variable name="primarySectionId">
        <xsl:apply-templates mode="id_or_generate_id" select="."/>
      </xsl:variable>
      
      <fo:page-sequence master-reference="simplePageLayout">
        <fo:static-content flow-name="xsl-region-before"> 
          <fo:block font-size="xx-large" color="red">Related exercises</fo:block>
          <fo:block font-size="xx-small" color="red" text-align-last='justify'>
            <fo:leader leader-pattern="rule" leader-length="100%"/>
          </fo:block>
        </fo:static-content>      
        
        <fo:flow flow-name="xsl-region-body">
          <xsl:for-each select="$followupQandasets">
            <fo:block>
              <xsl:apply-templates select="." mode="createTitleLink"/>
            </fo:block>                
          </xsl:for-each>
        </fo:flow>
      </fo:page-sequence>
    </xsl:if>
    
  </xsl:template>
  
  <xsl:template match="d:qandaset" mode="createTitleLink">
    
    <xsl:variable name="parentId" select="ancestor::d:part/@xml:id"/>      
        
    <fo:block>
      <xsl:text>Exercise </xsl:text>
      <xsl:value-of select="count(preceding::d:qandaset[ancestor::d:part/@xml:id = $parentId]) + 1"/>
      <xsl:text>: </xsl:text>
      <xsl:choose>
        <xsl:when test="d:title">        
          <xsl:apply-templates select="d:title/text()|d:title/*"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:for-each select="preceding-sibling::d:title[1]">
            <xsl:apply-templates/>
          </xsl:for-each>
        </xsl:otherwise>
      </xsl:choose>
    </fo:block>
  </xsl:template>
  
  <xsl:template match="d:xref|d:link" mode="createTitleLink">
    <xsl:apply-templates select="."/>
  </xsl:template>
  
  <xsl:template match="*|text()" mode="createTitleLink">
    <xsl:param name="linkTarget"/>
    <xsl:apply-templates select="."/>
  </xsl:template>
  
  <!-- Get rid of block object names e.g. "Figure 3.1" -->
  <xsl:template match="d:figure">
    <xsl:apply-templates/>
  </xsl:template>
  
  <!-- Ignore e.g. calloutlist elements inside figure --> 
  <xsl:template match="*[contains(@role, 'slideExclude')]">
    <xsl:message>
      <xsl:text>Excluding &lt;</xsl:text>
      <xsl:value-of select="name(.)"/>
      <xsl:text>&gt; from slides</xsl:text>
    </xsl:message>
  </xsl:template>
  
  
  <xsl:template match="*" mode="slideRootSections">
    <p>
      <xsl:text>No template defined for element '</xsl:text>
      <xsl:value-of select="name(.)"/>
      <xsl:text>' in mode 'slideRootSections'.</xsl:text>
    </p>
  </xsl:template>
  
  <xsl:template match="*" mode="slideUnit">
    <xsl:message>
      <xsl:text>No template matching '</xsl:text>
      <xsl:value-of select="name(.)"/>
      <xsl:text>' in mode 'slideUnit'</xsl:text>
    </xsl:message>    
  </xsl:template>
  
</xsl:stylesheet>
